package com.fjhx.victoriatourist.service.excess.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.victoriatourist.entity.excess.dto.ExcessGoodsDetailsDto;
import com.fjhx.victoriatourist.entity.excess.dto.ExcessGoodsDetailsSelectDto;
import com.fjhx.victoriatourist.entity.excess.po.ExcessGoodsDetails;
import com.fjhx.victoriatourist.entity.excess.po.ExcessGoodsRegister;
import com.fjhx.victoriatourist.entity.excess.vo.ExcessGoodsDetailsVo;
import com.fjhx.victoriatourist.mapper.excess.ExcessGoodsDetailsMapper;
import com.fjhx.victoriatourist.service.excess.ExcessGoodsDetailsService;
import com.fjhx.victoriatourist.service.excess.ExcessGoodsRegisterService;
import com.ruoyi.common.utils.wrapper.IWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 多货登记明细 服务实现类
 * </p>
 *
 * @author
 * @since 2024-03-27
 */
@Service
public class ExcessGoodsDetailsServiceImpl extends ServiceImpl<ExcessGoodsDetailsMapper, ExcessGoodsDetails> implements ExcessGoodsDetailsService {

    @Autowired
    private ExcessGoodsRegisterService excessGoodsRegisterService;

    @Autowired
    private ProductInfoService productInfoService;

    private IWrapper<ExcessGoodsDetails> getWrapper(ExcessGoodsDetailsSelectDto dto) {
        IWrapper<ExcessGoodsDetails> wrapper = getWrapper();
        wrapper.eq("egd", ExcessGoodsDetails::getStockWaitId, dto.getStockWaitId());
        wrapper.orderByDesc("egd", ExcessGoodsDetails::getId);
        return wrapper;
    }

    @Override
    public List<ExcessGoodsDetailsVo> getList(ExcessGoodsDetailsSelectDto dto) {
        IWrapper<ExcessGoodsDetails> wrapper = getWrapper(dto);
        List<ExcessGoodsDetailsVo> list = this.baseMapper.getList(wrapper);
        setInfo(list);
        return list;
    }

    @Override
    public Page<ExcessGoodsDetailsVo> getPage(ExcessGoodsDetailsSelectDto dto) {
        IWrapper<ExcessGoodsDetails> wrapper = getWrapper(dto);
        Page<ExcessGoodsDetailsVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<ExcessGoodsDetailsVo> records = page.getRecords();
        if (ObjectUtil.isEmpty(records)) {
            return page;
        }
        return page;
    }

    void setInfo(List<ExcessGoodsDetailsVo> list) {
        List<Long> ids = list.stream().map(ExcessGoodsDetails::getId).collect(Collectors.toList());
        if (ObjectUtil.isEmpty(ids)) {
            return;
        }
        //赋值清点列表
        Map<Long, List<ExcessGoodsRegister>> longListMap = excessGoodsRegisterService.mapKGroup(ExcessGoodsRegister::getExcessGoodsDetailsId, q -> q
                .in(ExcessGoodsRegister::getExcessGoodsDetailsId, ids)
        );

        for (ExcessGoodsDetailsVo excessGoodsDetailsVo : list) {
            excessGoodsDetailsVo.setExcessGoodsRegisterList(longListMap.getOrDefault(excessGoodsDetailsVo.getId(), new ArrayList<>()));
        }

        //赋值产品信息
        productInfoService.attributeAssign(list, ExcessGoodsDetails::getProductId, (item, productinfo) -> {
            item.setProductCode(productinfo.getCode());
            item.setProductCustomCode(productinfo.getCustomCode());
            item.setProductName(productinfo.getName());
            item.setProductSpec(productinfo.getSpec());
            item.setProductUnit(productinfo.getUnit());
        });
    }

    @Override
    public ExcessGoodsDetailsVo detail(Long id) {
        ExcessGoodsDetails ExcessGoodsDetails = this.getById(id);
        ExcessGoodsDetailsVo result = BeanUtil.toBean(ExcessGoodsDetails, ExcessGoodsDetailsVo.class);
        return result;
    }

    @Override
    public void add(ExcessGoodsDetailsDto excessGoodsDetailsDto) {
        this.save(excessGoodsDetailsDto);
    }

    @Override
    public void edit(ExcessGoodsDetailsDto excessGoodsDetailsDto) {
        this.updateById(excessGoodsDetailsDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
